import { Controller, CustomAttribute, CustomElement } from "aurelia";
import { strings } from "@material/dialog";
import { MdcRipple } from "@aurelia2-mdc-web/ripple";
/** Service to open MDC dialogs */
export class MdcDialogService {
    container;
    constructor(container) {
        this.container = container;
    }
    /** Opens the dialog specified in the options */
    async open(options) {
        const host = document.createElement("div");
        document.body.append(host);
        const ViewModelClass = options.viewModel;
        const model = options.model;
        await this.attachViewToDom(host, ViewModelClass, model);
        const dialogController = CustomElement.for(host.querySelector(".mdc-dialog"));
        const dialogVm = dialogController.viewModel;
        if (!dialogVm) {
            throw new Error("MDC-DIALOG element is missing");
        }
        let openedResolver;
        const openedPromise = new Promise(r => openedResolver = r);
        let opened = false;
        dialogVm.root.addEventListener(strings.OPENED_EVENT, () => {
            opened = true;
            openedResolver();
        });
        let closedResolver;
        const closedPromise = new Promise(r => closedResolver = r);
        dialogVm.root.addEventListener(strings.CLOSED_EVENT, (evt) => {
            if (!opened) {
                // The dialog was closed before it was opened.
                // Emit the event to prevent an unresolved open promise.
                dialogVm.emit(strings.OPENED_EVENT, {});
            }
            closedResolver(evt.detail.action);
        });
        await dialogVm.initialised;
        dialogVm.open();
        await openedPromise;
        // re-layout ripple elements because dialogs use `transform: scale(.8)` and initial layout is incorrect
        const ripples = Array.from(dialogVm.root.querySelectorAll(`.${MdcRipple.ATTRIBUTE_CLASS}`));
        await Promise.all(ripples.map(async (x) => {
            const attr = CustomAttribute.for(x, 'mdc-ripple');
            await attr?.viewModel.initialised;
            attr?.viewModel.foundation?.layout();
        }));
        const action = await closedPromise;
        return action;
    }
    async removeViewFromDom(_, component) {
        const customElement = CustomElement.for(component);
        await customElement.deactivate(customElement, null);
    }
    async attachViewToDom(parentElement, component, model) {
        const c = this.container.invoke(component);
        await c.activate?.(model);
        const definition = CustomElement.getDefinition(component);
        const controller = Controller.$el(this.container, c, parentElement, null, definition);
        await controller.activate(controller, null);
        return controller.host;
    }
}
//# sourceMappingURL=mdc-dialog-service.js.map